@@ -2,26 +2,27 @@ package Mojolicious::Plugin::AssetPack;
 
 use Mojo::Base 'Mojolicious::Plugin';
 use Mojo::ByteStream;
+use Mojo::JSON ();
 use Mojo::Util ();
 use Mojolicious::Plugin::AssetPack::Asset;
 use Mojolicious::Plugin::AssetPack::Preprocessors;
-use Cwd            ();
-use File::Basename ();
-use File::Path     ();
-use File::Spec     ();
+use Cwd ();
+use File::Basename qw( basename );
+use File::Path ();
+use File::Spec::Functions qw( catdir catfile );
 use constant DEBUG    => $ENV{MOJO_ASSETPACK_DEBUG}    || 0;
 use constant NO_CACHE => $ENV{MOJO_ASSETPACK_NO_CACHE} || 0;
 
-our $VERSION = '0.64';
+our $VERSION = '0.65';
+
+our $MINIFY = undef;    # internal use only!
+my $MONIKER_RE = qr{^(.+)\.(\w+)$};
 
 has base_url      => '/packed/';
-has headers       => sub { +{} };
 has minify        => 0;
 has preprocessors => sub { Mojolicious::Plugin::AssetPack::Preprocessors->new };
-has out_dir       => '';
 
-has _app => undef;
-has _ua  => sub {
+has _ua => sub {
   require Mojo::UserAgent;
   Mojo::UserAgent->new(max_redirects => 3);
 };
@@ -30,37 +31,44 @@ sub add {
   my ($self, $moniker, @files) = @_;
 
   @files = $self->_expand_wildcards(@files);
-  return $self->tap(sub { $self->{files}{$moniker} = \@files }) if NO_CACHE;
-  return $self->tap(sub { $self->_assets($moniker => $self->_process($moniker, @files)) }) if $self->minify;
-  return $self->tap(sub { $self->_assets($moniker => $self->_process_many($moniker, @files)) });
+  return $self->tap(sub { $self->{files}{$moniker} = \@files; $self }) if NO_CACHE;
+  return $self->tap(sub { $self->_processed($moniker, $self->_process($moniker, @files)) }) if $self->minify;
+  return $self->tap(sub { $self->_processed($moniker, $self->_process_many($moniker, @files)) });
 }
 
 sub fetch {
   my $self  = shift;
   my $url   = Mojo::URL->new(shift);
-  my $asset = $self->_packed("$url")
-    || $self->_handler($url->scheme)->asset_for($url, $self)->in_memory($self->out_dir ? 0 : 1)->save;
+  my $asset = $self->_handler($url->scheme)->asset_for($url, $self);
   return $asset if @_;    # internal
   return $asset->path;    # documented api
 }
 
 sub get {
   my ($self, $moniker, $args) = @_;
-  my $assets = $self->_assets($moniker);
+  my @assets = $self->_processed($moniker);
 
-  die "Asset '$moniker' is not defined." unless @$assets;
-  return @$assets if $args->{assets};
-  return map { $_->slurp } @$assets if $args->{inline};
-  return map { $self->base_url . $_->basename } @$assets;
+  return @assets if $args->{assets};
+  return map { $_->slurp } @assets if $args->{inline};
+  return map { $self->base_url . basename($_->path) } @assets;
 }
 
-sub preprocessor {
-  my ($self, $name, $args) = @_;
-  $args->{extensions} or die "Usage: \$self->preprocessor(\$name => {extensions => [...]})";
-  $self->preprocessors->add($_ => $name => $args) for @{$args->{extensions}};
-  return $self;
+sub headers {
+  my ($self, $headers) = @_;
+
+  $self->_app->hook(
+    after_static => sub {
+      my $c    = shift;
+      my $path = $c->req->url->path->canonicalize;
+      return unless $path->[1] and 0 == index "$path", $self->base_url;
+      my $h = $c->res->headers;
+      $h->header($_ => $headers->{$_}) for keys %$headers;
+    }
+  );
 }
 
+sub out_dir { shift->{out_dir} }
+
 sub purge {
   my ($self, $args) = @_;
   my $file_re = $self->minify ? qr/^(.*?)-(\w{32})\.min\.(\w+)$/ : qr/^(.*?)-(\w{32})\.(\w+)$/;
@@ -70,14 +78,14 @@ sub purge {
   local $args->{always} = $args->{always} // $self->_app->mode eq 'development';
 
   return $self unless $args->{always};
-  die '$app->asset->purge() must be called AFTER $app->asset(...)' unless keys %{$self->{assets} || {}};
-  return $self unless $self->out_dir and opendir $PACKED, $self->out_dir;
+  die '$app->asset->purge() must be called AFTER $app->asset(...)' unless keys %{$self->{asset} || {}};
+  return $self unless -w $self->out_dir and opendir $PACKED, $self->out_dir;
   $existing{$_} = 1 for grep { $_ =~ $file_re } readdir $PACKED;
-  delete $existing{$_->basename} for map {@$_} values %{$self->{assets} || {}};
+  delete $existing{$_} for map { basename $_->path } values %{$self->{asset} || {}};
 
   for my $file (keys %existing) {
-    $self->_unlink_packed($file);
-    warn "[ASSETPACK] Purged $file ($!)\n" if DEBUG;
+    unlink catfile $self->out_dir, $file;
+    $self->_app->log->debug("AssetPack purge $file: @{[$! || 'Deleted']}");
   }
 
   return $self;
@@ -90,30 +98,22 @@ sub register {
   if (eval { $app->$helper }) {
     return $app->log->debug("AssetPack: Helper $helper() is already registered.");
   }
+  if (my $paths = $config->{source_paths}) {
+    $self->{source_paths} = [map { -d $_ ? Cwd::abs_path($_) : $app->home->rel_file($_) } @$paths];
+  }
 
-  $self->{assets}       = {};
-  $self->{processed}    = {};
-  $self->{source_paths} = $self->_build_source_paths($app, $config);
-
-  # Not official. Want to keep it private for now...
   $self->{die_on_process_error} = $ENV{MOJO_ASSETPACK_DIE_ON_PROCESS_ERROR} // $app->mode ne 'development';
+  $self->{map_file} = '_assetpack.map';
 
-  $self->_app($app);
   $self->_ua->server->app($app);
-  $self->_ua->proxy->detect if $config->{proxy};
-  $self->headers($config->{headers} || {});
-  $self->minify($config->{minify} // $app->mode ne 'development');
-  $self->out_dir($self->_build_out_dir($app, $config));
-  $self->base_url($config->{base_url}) if $config->{base_url};
-  $self->_reloader($app, $config->{reloader}) if $config->{reloader};
+  Scalar::Util::weaken($self->_ua->server->{app});
 
-  if (NO_CACHE) {
-    $app->log->info('AssetPack Will rebuild assets on each request in memory');
-    $self->out_dir('');
-  }
-  if (!$self->out_dir) {
-    $self->{assets_from_memory} = 1;
-  }
+  $self->_ua->proxy->detect if $config->{proxy};
+  $self->headers($config->{headers}) if $config->{headers};
+  $self->minify($MINIFY // $config->{minify} // $app->mode ne 'development');
+  $self->base_url($config->{base_url}) if $config->{base_url};
+  $self->_build_out_dir($app, $config);
+  $self->_load_mapping;
 
   $app->helper(
     $helper => sub {
@@ -122,95 +122,66 @@ sub register {
       return $self->_inject(@_);
     }
   );
-
-  Mojo::IOLoop->next_tick(sub { $self->_add_hook($app, $config) });
 }
 
 sub source_paths {
   my $self = shift;
-  my $app  = $self->_app;
-
-  if (@_) {
-    $self->{source_paths} = shift;
-    return $self;
-  }
-  else {
-    return $self->{source_paths} || return $app->static->paths;
-  }
+  return $self->{source_paths} || $self->_app->static->paths unless @_;
+  $self->{source_paths} = shift;
+  return $self;
 }
 
-sub _add_hook {
-  my ($self, $app, $config) = @_;
-  my $headers = $self->headers;
+sub test_app {
+  my ($class, $app) = @_;
+  my $n = 0;
 
-  if ($self->{assets_from_memory}) {
-    $app->hook(
-      before_routes => sub {
-        my $c    = shift;
-        my $path = $c->req->url->path;
+  require Test::Mojo;
 
-        return if $c->req->is_handshake or $c->res->code;
-        return unless $path->[1] and 0 == index "$path", $self->base_url;
-        return unless my $asset = $c->asset->{asset}{$path->[1]};
-        return if $asset->{internal};
-        my $h = $c->res->headers->last_modified(Mojo::Date->new($^T))
-          ->content_type($c->app->types->type($asset->path =~ /\.(\w+)$/ ? $1 : 'txt') || 'text/plain');
-        $h->header($_ => $headers->{$_}) for keys %$headers;
-        $c->reply->asset($asset);
-      }
-    );
-  }
-  elsif (%$headers) {
-    $app->hook(
-      after_static => sub {
-        my $c    = shift;
-        my $path = $c->req->url->path;
-        return unless $path->[1] and 0 == index "$path", $self->base_url;
-        my $h = $c->res->headers;
-        $h->header($_ => $headers->{$_}) for keys %$headers;
-      }
-    );
+  for my $m (0, 1) {
+    Test::More::diag("minify=$m") if DEBUG;
+    local $MINIFY = $m;
+    my $t = Test::Mojo->new($app);
+    my $processed = $t->app->asset->{processed} or next;
+    for my $asset (map {@$_} values %$processed) {
+      $t->get_ok("/packed/$asset")->status_is(200);
+      $n++;
+    }
+    Test::More::ok($n, "Generated $n assets for $app with minify=$m");
   }
+
+  return $class;
 }
 
+sub _app { shift->_ua->server->app }
+
 sub _asset {
   my ($self, $name) = @_;
-  my $asset = $self->{asset}{$name} ||= Mojolicious::Plugin::AssetPack::Asset->new;
-  $asset->path(File::Spec->catfile($self->out_dir, $name)) unless $asset->path;
-  $asset;
-}
-
-sub _assets {
-  my ($self, $moniker, @assets) = @_;
-  $self->{assets}{$moniker} = \@assets if @assets;
-  $self->{assets}{$moniker} || [];
+  $self->{asset}{$name} ||= Mojolicious::Plugin::AssetPack::Asset->new(path => catfile $self->out_dir, $name);
 }
 
 sub _build_out_dir {
   my ($self, $app, $config) = @_;
-  my $out_dir = $config->{out_dir};
+  my $out_dir;
 
-  if ($out_dir) {
-    my $static_dir = Cwd::abs_path(File::Spec->catdir($out_dir, File::Spec->updir));
+  if ($out_dir = $config->{out_dir}) {
+    my $static_dir = Cwd::abs_path(catdir $out_dir, File::Spec->updir);
     push @{$app->static->paths}, $static_dir unless grep { $_ eq $static_dir } @{$app->static->paths};
   }
-  elsif (!defined $out_dir) {
+  if (!$out_dir) {
     for my $path (@{$app->static->paths}) {
-      next unless -w $path;
-      $out_dir = File::Spec->catdir($path, 'packed');
-      last;
+      my $packed = catdir $path, 'packed';
+      if (-w $path) { $out_dir = Cwd::abs_path($packed); last }
+      if (-r $packed) { $out_dir ||= Cwd::abs_path($packed) }
     }
   }
+  if (!$out_dir) {
+    die "[AssetPack] Could not auto detect out_dir: "
+      . "Neither readable, nor writeable 'packed' directory could be found in static paths, @{$app->static->paths}. Maybe you forgot to pre-pack the assets? "
+      . "https://metacpan.org/pod/Mojolicious::Plugin::AssetPack::Manual::Cookbook";
+  }
 
-  File::Path::make_path($out_dir) if $out_dir and !-d $out_dir;
-  return $out_dir // '';
-}
-
-sub _build_source_paths {
-  my ($self, $app, $config) = @_;
-
-  return undef unless my $paths = $config->{source_paths};
-  return [map { -d $_ ? Cwd::abs_path($_) : $app->home->rel_file($_) } @$paths];
+  File::Path::make_path($out_dir) unless -d $out_dir;
+  $self->{out_dir} = $out_dir;
 }
 
 sub _expand_wildcards {
@@ -222,7 +193,7 @@ sub _expand_wildcards {
       my @rel = split '/', $file;
       my $glob = pop @rel;
 
-      for my $path (map { File::Spec->catdir($_, @rel) } @{$self->source_paths}) {
+      for my $path (map { catdir $_, @rel } @{$self->source_paths}) {
         my $cwd = Mojolicious::Plugin::AssetPack::Preprocessors::CWD->new($path);
         push @files, grep { !$seen{$_} } map { join '/', @rel, $_ } sort glob $glob;
       }
@@ -236,6 +207,28 @@ sub _expand_wildcards {
   return @files;
 }
 
+sub _handle_process_error {
+  my ($self, $moniker, $err) = @_;
+  my ($name, $ext) = $moniker =~ $MONIKER_RE;
+  my $app          = $self->_app;
+  my $source_paths = join ',', @{$self->source_paths};
+  my $static_paths = join ',', @{$app->static->paths};
+  my $msg;
+
+  $err =~ s!\s+$!!;    # remove newlines
+  $msg = "$err {source_paths=[$source_paths], static_paths=[$static_paths]}";
+
+  # use fixed mapping
+  if (my @assets = $self->_processed($moniker)) {
+    $app->log->debug("AssetPack falls back to predefined mapping on: $msg");
+    return @assets;
+  }
+
+  $app->log->error($msg);
+  die $msg if $self->{die_on_process_error};    # EXPERIMENTAL: Prevent hot reloading when assetpack fail
+  return $self->_asset("$name-err.$ext")->_spurt_error_message_for($ext, $err);
+}
+
 sub _handler {
   my ($self, $moniker) = @_;
   $self->{handler}{$moniker} ||= do {
@@ -249,57 +242,41 @@ sub _inject {
   my ($self, $c, $moniker, $args, @attrs) = @_;
   my $tag_helper = $moniker =~ /\.js/ ? 'javascript' : 'stylesheet';
 
-  if (NO_CACHE) {
-    $self->_assets($moniker => $self->_process_many($moniker, @{$self->{files}{$moniker} || []}));
-  }
+  NO_CACHE and $self->_processed($moniker, $self->_process_many($moniker, @{$self->{files}{$moniker} || []}));
+
+  return Mojo::ByteStream->new(qq(<!-- Asset '$moniker' is not defined\. -->))
+    unless my @res = $self->get($moniker, $args);
+  return $c->$tag_helper(@attrs, sub { join '', @res }) if $args->{inline};
+  return Mojo::ByteStream->new(join "\n", map { $c->$tag_helper($_, @attrs) } @res);
+}
+
+sub _load_mapping {
+  my $self = shift;
+  my $mode = $self->minify ? 'min' : 'normal';
+
+  $self->{mapping} = {normal => {}, min => {}, meta => {ts => time}};
+  $self->{processed} = $self->{mapping}{$mode};
 
   eval {
-    if ($args->{inline}) {
-      return $c->$tag_helper(@attrs, sub { join '', $self->get($moniker, $args) });
+    my $file = catfile $self->out_dir, $self->{map_file};
+    my $mapping = Mojo::JSON::decode_json(Mojo::Util::slurp($file));
+    for my $mode (keys %$mapping) {
+      $self->{mapping}{$mode}{$_} ||= $mapping->{$mode}{$_} for keys %{$mapping->{$mode}};
     }
-    else {
-      return Mojo::ByteStream->new(join "\n", map { $c->$tag_helper($_, @attrs) } $self->get($moniker, $args));
-    }
-    1;
-  } or do {
-    $self->_app->log->error($@);
-    return Mojo::ByteStream->new(qq(<!-- Asset '$moniker' is not defined\. -->));
   };
 }
 
-sub _make_error_asset {
-  my ($self, $moniker, $file, $err) = @_;
-
-  $err =~ s!\r!!g;
-  $err =~ s!\n+$!!;
-  $err = "$file: $err";
-
-  if ($moniker =~ /\.js$/) {
-    $err =~ s!'!"!g;
-    $err =~ s!\n!\\n!g;
-    $err =~ s!\s! !g;
-    return "alert('$err');console.log('$err');";
-  }
-  else {
-    $err =~ s!"!'!g;
-    $err =~ s!\n!\\A!g;
-    $err =~ s!\s! !g;
-    return
-      qq(html:before{background:#f00;color:#fff;font-size:14pt;position:fixed;padding:20px;z-index:9999;content:"$err";});
-  }
-}
-
 sub _packed {
-  my $self = shift;
-  my $needle = ref $_[0] ? shift : _name(shift);
+  my $sorter = ref $_[-1] eq 'CODE' ? pop : sub {@_};
+  my ($self, $needle) = @_;
 
-  for my $path (map { File::Spec->catdir($_, 'packed') } @{$self->_app->static->paths}) {
-    opendir my $DH, $path or next;
-    for (readdir $DH) {
-      next unless /$needle/;
-      $path = Cwd::abs_path(File::Spec->catfile($path, $_));
-      $self->_app->log->debug("Using existing asset $path") if DEBUG;
-      return $self->_asset($_)->path($path)->in_memory(0);
+  for my $dir (map { catdir $_, 'packed' } @{$self->_app->static->paths}) {
+    opendir my $DH, $dir or next;
+    for my $file ($sorter->(map { catfile $dir, $_ } readdir $DH)) {
+      my $name = basename $file;
+      next unless $name =~ $needle;
+      $self->_app->log->debug("Using existing asset $file") if DEBUG;
+      return $self->_asset($name)->path($file);
     }
   }
 
@@ -308,9 +285,8 @@ sub _packed {
 
 sub _process {
   my ($self, $moniker, @sources) = @_;
-  my $app   = $self->_app;
   my $topic = $moniker;
-  my ($name, $ext) = (_name($moniker), _ext($moniker));
+  my ($name, $ext) = $moniker =~ $MONIKER_RE;
   my ($asset, $file, @checksum);
 
   eval {
@@ -318,6 +294,7 @@ sub _process {
       $topic = $s;
       $s     = $self->_source_for_url($s);    # rewrite @sources
       push @checksum, $self->preprocessors->checksum(_ext($topic), \$s->slurp, $s->path);
+      warn sprintf "[AssetPack] Checksum $checksum[-1] from %s\n", $s->path if DEBUG;
     }
 
     @checksum = (Mojo::Util::md5_sum(join '', @checksum)) if @checksum > 1;
@@ -325,96 +302,79 @@ sub _process {
     return $asset if $asset;                  # already processed
 
     $file = $self->minify ? "$name-$checksum[0].min.$ext" : "$name-$checksum[0].$ext";
-    $asset = $self->{asset}{$file} = Mojolicious::Plugin::AssetPack::Asset->new;
-    $asset->in_memory(1)->path(File::Spec->catfile($self->out_dir, $file));
+    $asset = $self->_asset($file);
+    warn sprintf "[AssetPack] Creating %s from %s\n", $file, join ', ', map { $_->path } @sources if DEBUG;
 
     for my $s (@sources) {
-      $topic = $s->basename;
+      $topic = basename($s->path);
       my $content = $s->slurp;
       $self->preprocessors->process(_ext($s->path), $self, \$content, $s->path);
-      $asset->content($asset->content . $content);
+      $asset->add_chunk($content);
     }
 
-    $asset->in_memory($self->out_dir ? 0 : 1)->save;
-  } or do {
-    my $err          = $@;
-    my $source_paths = join ',', @{$self->source_paths};
-    my $static_paths = join ',', @{$app->static->paths};
-    $err =~ s!\s+$!!;    # remove newlines
-    my $msg = "AssetPack failed to process $topic: $err {source_paths=[$source_paths], static_paths=[$static_paths]}";
-    $app->log->error($msg);
-    die $msg if $self->{die_on_process_error};    # prevent hot reloading when assetpack fail
-    $file ||= "$name-@{[time]}.$ext";
-    $file =~ s!$ext$!err.$ext!;
-    $asset = $self->{asset}{$file} = Mojolicious::Plugin::AssetPack::Asset->new(in_memory => 1, path => $file);
-    $asset->content($self->_make_error_asset($moniker, $topic, $err || 'Unknown error'));
+    $self->{changed}++ unless $self->{processed}{$moniker} and $self->{processed}{$moniker}[0] eq $file;
+    $self->{processed}{$moniker} = [$file];
+    $self->_app->log->info("AssetPack built @{[$asset->path]} for @{[$self->_app->moniker]}.");
   };
 
-  if ($asset->in_memory) {
-    $app->log->info("AssetPack built @{[$asset->path]} for @{[$app->moniker]} in memory.");
-    $self->{assets_from_memory}++;
-  }
-  elsif ($file) {
-    $app->log->info("AssetPack built @{[$asset->path]} for @{[$app->moniker]}.");
-  }
-
-  return $asset;
+  return $asset unless $@;
+  return $self->_handle_process_error($moniker, "AssetPack could not read $topic: $@") unless $file;
+  return $self->_handle_process_error($moniker, "AssetPack could not generate $file ($topic): $@");
 }
 
 sub _process_many {
   my ($self, $moniker, @files) = @_;
   my $ext = _ext($moniker);
-  map { my $name = _name($_); $self->_process("$name.$ext" => $_) } @files;
+
+  return map {
+    my $topic = $_;
+    local $_ = $topic;    # do not modify input
+    s![^\w-]!_!g if /^https?:/;
+    s!\.\w+$!!;
+    $_ = basename $_;
+    $self->_process("$_.$ext" => $topic);
+  } @files;
 }
 
-sub _reloader {
-  my ($self, $app, $config) = @_;
-  my $reloader = $self->_asset('reloader.js');
-
-  return if !$config->{enabled} and $app->mode ne 'development';
-
-  warn "[ASSETPACK] Adding reloader asset and route\n" if DEBUG;
-  $reloader->path('reloader.js')->{internal} = 1;
-  $self->{assets}{'reloader.js'} = [$reloader];
-  push @{$app->renderer->classes}, __PACKAGE__;
-  $app->routes->get('/packed/reloader')->to(template => 'packed/reloader', strategy => 'document', %$config);
-  $app->routes->websocket('/packed/reloader/ws')->to(
-    cb => sub {
-      shift->on(message => sub { shift->send('pong'); });
-    }
-  )->name('assetpack.ws');
+sub _processed {
+  my ($self, $moniker, @assets) = @_;
+  return map { $self->_asset($_) } @{$self->{processed}{$moniker} || []} unless @assets;
+  $self->{processed}{$moniker} = [map { basename $_->path } @assets];
+  Mojo::Util::spurt(Mojo::JSON::encode_json($self->{mapping}), catfile $self->out_dir, $self->{map_file})
+    if $self->{changed};
+  return $self;
 }
 
 sub _source_for_url {
   my ($self, $url) = @_;
 
+  if ($self->{asset}{$url}) {
+    warn "[AssetPack] Asset already loaded: $url\n" if DEBUG;
+    return $self->{asset}{$url};
+  }
   if (my $scheme = Mojo::URL->new($url)->scheme) {
+    warn "[AssetPack] Asset from online resource: $url\n" if DEBUG;
     return $self->fetch($url, 'internal');
   }
 
   my @look_in = (@{$self->source_paths}, @{$self->_app->static->paths});
   my @path = split '/', $url;
 
-  for my $path (map { Cwd::abs_path(File::Spec->catfile($_, @path)) } @look_in) {
-    return $self->_asset($path)->path($path)->in_memory(0) if $path and -r $path;
+  for my $file (map { catfile $_, @path } @look_in) {
+    next unless $file and -r $file;
+    warn "[AssetPack] Asset from disk: $url ($file)\n" if DEBUG;
+    return $self->_asset("$url")->path($file);
   }
 
+  warn "[AssetPack] Asset from @{[$self->_app->moniker]}: $url\n" if DEBUG;
   return $self->_handler('https')->asset_for($url, $self);
 }
 
-sub _unlink_packed {
-  my ($self, $file) = @_;
-  unlink File::Spec->catfile($self->out_dir, $file);
-}
-
 # utils
-sub _ext { local $_ = File::Basename::basename($_[0]); /\.(\w+)$/ ? $1 : 'unknown'; }
+sub _ext { local $_ = basename $_[0]; /\.(\w+)$/ ? $1 : 'unknown'; }
 
-sub _name {
-  local $_ = $_[0];
-  return do { s![^\w-]!_!g; $_ } if /^https?:/;
-  $_ = File::Basename::basename($_);
-  /^(.*)\./ ? $1 : $_;
+sub _sort_by_mtime {
+  map { $_->[0] } sort { $b->[1] <=> $a->[1] } map { [$_, (stat $_)[9]] } @_;
 }
 
 1;
@@ -427,7 +387,7 @@ Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass, javascrip
 
 =head1 VERSION
 
-0.64
+0.65
 
 =head1 SYNOPSIS
 
@@ -438,9 +398,16 @@ Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass, javascrip
   # load plugin
   plugin "AssetPack";
 
-  # define assets: $moniker => @real_assets
+  # Define assets: $moniker => @real_assets
   app->asset('app.js' => '/js/foo.js', '/js/bar.js', '/js/baz.coffee');
-  app->asset->purge; # remove old packed files
+
+  # Add custom response headers for assets
+  app->asset->headers({"Cache-Control" => "max-age=31536000"});
+
+  # Remove old assets
+  app->asset->purge;
+
+  # Start the application
   app->start;
 
 See also L<Mojolicious::Plugin::AssetPack::Manual::Assets> for more
@@ -493,7 +460,7 @@ in different modes.
 See L<Mojolicious::Plugin::AssetPack::Manual::CustomDomain> for how to
 serve your assets from a custom host.
 
-=item * 
+=item *
 
 See L<Mojolicious::Plugin::AssetPack::Preprocessors> for details on the
 different (official) preprocessors.
@@ -549,17 +516,6 @@ See L<Mojolicious::Plugin::AssetPack::Manual::CustomDomain> for more details.
 
 NOTE! You need to have a trailing "/" at the end of the string.
 
-=head2 headers
-
-  $app->plugin("AssetPack" => {headers => {"Cache-Control" => "max-age=31536000"}});
-  $hash_ref = $self->headers;
-
-This attribute can hold custom response headers when serving assets.
-The default is no headers, but this might change in future release to
-include "Cache-Control".
-
-This attribute is EXPERIMENTAL.
-
 =head2 minify
 
   $app->plugin("AssetPack" => {minify => $bool});
@@ -571,16 +527,6 @@ Default is false in "development" L<mode|Mojolicious/mode> and true otherwise.
 
 See also L<Mojolicious::Plugin::AssetPack::Manual::Modes>.
 
-=head2 out_dir
-
-  $app->plugin("AssetPack" => {out_dir => $str});
-  $str = $self->out_dir;
-
-Holds the path to the directory where packed files can be written.
-
-Defaults to empty string if no directory can be found, which again results in
-keeping all packed files in memory.
-
 =head2 preprocessors
 
   $obj = $self->preprocessors;
@@ -615,11 +561,25 @@ contain one file if L</minify> is true.
 See L<Mojolicious::Plugin::AssetPack::Manual::Include/Full control> for mode
 details.
 
-=head2 preprocessor
+=head2 headers
 
-DEPRECATED. Use this instead:
+  $app->plugin("AssetPack" => {headers => {"Cache-Control" => "max-age=31536000"}});
+  $app->asset->headers({"Cache-Control" => "max-age=31536000"});
 
-  $self->preprocessors->add($extension => $class => \%attrs);
+Calling this method will add a L<after_static|Mojolicious/after_static> hook which
+will set additional response headers when an asset is served.
+
+This method is EXPERIMENTAL.
+
+=head2 out_dir
+
+  $app->plugin("AssetPack" => {out_dir => $str});
+  $str = $self->out_dir;
+
+Returns the path to the directory where generated packed files are located.
+
+Changing this from the default will probably lead to inconsistency. Please
+report back if you are using this feature with success.
 
 =head2 purge
 
@@ -660,6 +620,16 @@ for more information.
 
 This method is EXPERIMENTAL and can change, but will most likely not be removed.
 
+=head2 test_app
+
+  Mojolicious::Plugin::AssetPack->test_app("MyApp");
+
+This method will create two L<Mojo::Test> instances of "MyApp" and create
+assets with L</minify> set to 0 and 1.
+L<Mojolicious::Plugin::AssetPack::Manual::Cookbook/SHIPPING> for more details.
+
+This method is EXPERIMENTAL.
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2014, Jan Henning Thorsen.
@@ -678,37 +648,3 @@ Per Edin - C<info@peredin.com>
 Viktor Turskyi
 
 =cut
-
-__DATA__
-@@ packed/reloader.js.ep
-;window.addEventListener('load', function(e) {
-  var xhr, socket, t, reloaded = 0;
-  var connect = function() {
-    socket = new WebSocket('<%= url_for('assetpack.ws')->userinfo(undef)->to_abs %>'.replace(/^http/, 'ws'));
-    socket.onopen = function(e) {
-      if (reloaded++) {
-        xhr = new XMLHttpRequest();
-        xhr.responseType = 'document';
-        xhr.open('GET', window.location.href);
-        xhr.onreadystatechange = function() {
-          if (xhr.readyState != 4) return;
-          if (window.console) console.log('[AssetPack] Replacing <head>...</head>');
-          document.head.innerHTML = this.responseXML.getElementsByTagName('head')[0].innerHTML;
-        };
-        xhr.send(null);
-      }
-      t = setInterval(function() { socket.send('ping'); }, 5000);
-    }
-    socket.onclose = function() {
-      if (t) clearTimeout(t);
-      if (window.console) console.log('[AssetPack] Reloading with strategy "<%= $strategy %>" (' + reloaded + ')');
-      if ('<%= $strategy %>' == 'document') {
-        return window.location = window.location.href;
-      }
-      else {
-        setTimeout(function() { connect() }, 500);
-      }
-    };
-  };
-  connect();
-});
