@@ -66,7 +66,7 @@ sub asset_for {
     $cn =~ s!\.\w+$!!;
     $cn =~ s!\W!-!g;
     $css .= ".$name.$cn { background-position: 0px -$size[1]px; width: ${w}px; height: ${h}px; }\n";
-    warn "[ASSETPACK] Adding $directory/$file to sprite $name.png $w x $h\n" if DEBUG;
+    warn "[AssetPack] Adding $directory/$file to sprite $name.png $w x $h\n" if DEBUG;
     $tiled->paste(src => $tile, left => 0, top => $size[1]) or die $tiled->errstr;
     $size[1] += $h;
     $size[0] = $w if $size[0] < $w;
@@ -75,8 +75,8 @@ sub asset_for {
   $tiled->crop(right => $size[0], bottom => $size[1])->write(data => \my $sprite, type => 'png') or die $tiled->errstr;
   $checksum = Mojo::Util::md5_sum($sprite);
   $css .= ".$name { background: url($name-$checksum.png) no-repeat; display: inline-block; }\n";
-  $assetpack->_asset("$name-$checksum.png")->content($sprite)->in_memory($assetpack->out_dir ? 0 : 1)->save;
-  $assetpack->_asset("$name.css")->content($css);
+  $assetpack->_asset("$name-$checksum.png")->spurt($sprite);
+  $assetpack->_asset("$name.css")->spurt($css);
 }
 
 sub _imager {
